/* $Id: algo_int.h,v 1.1 2002/01/24 15:15:16 levitte Exp $ */

/* ====================================================================
 * Copyright (c) 1999 Levitte Programming.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *      This product includes software developed by Levitte Programming
 *      and its contributors.
 *
 * 4. The names "RLCRYPTO" and "Levitte Programming" or the names of its
 *    contributors must not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *    For written permission, please contact levitte@lp.se.
 *
 * THIS SOFTWARE IS PROVIDED BY LEVITTE PROGRAMMING AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL LEVITTE
 * PROGRAMMING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

#ifndef HEADER_ALGO_INT_H
#define HEADER_ALGO_INT_H

#include <rlcrypto/algorithms.h>

#ifdef _cplusplus
extern "C"
{
#endif

  struct symmetric_method_st
  {
    const char *name;		/* just a descriptive name */
    int nid;			/* If there's a NID for it */
    size_t key_bits;		/* Set to -1 if variable */
    size_t block_bits;
    int locked;			/* When this lock is set, all fields
				   have been filled in properly. */
    size_t (*algo_ctx_size)(void);
    ALGO_CTX *(*alloc)(void);
    void (*free)(ALGO_CTX *ctx);
    int (*init)(ALGO_CTX *ctx, const unsigned char *inkey, size_t keylen);
    void (*clear)(ALGO_CTX *ctx);
    int (*enc_dec)(ALGO_CTX *ctx,
		   const unsigned char *input, unsigned char *output, size_t length,
		   int encrypt);
  };

  struct symmetric_mode_method_st {
    const char *name;		/* just a descriptive name */
    int nid;			/* If there's a NID for it */
    int locked;			/* When this lock is set, all fields
				   have been filled in properly. */
    size_t (*algo_ctx_size)(void);
    ALGO_CTX *(*alloc)(void);
    void (*free)(ALGO_CTX *ctx);
    int (*init)(ALGO_CTX *ctx, const unsigned char *ivec, size_t iveclen);
    void (*clear)(ALGO_CTX *ctx);
    int (*enc_dec)(ALGO_CTX *ctx,
		   const unsigned char *input, unsigned char *output, size_t length,
		   int encrypt,
		   int (*sym_enc_dec)(ALGO_CTX *ctx,
				      const unsigned char *input,
				      unsigned char *output, size_t length,
				      int encrypt),
		   ALGO_CTX *sym_data);
    int (*cpy_ivec)(ALGO_CTX *ctx, unsigned char *ivec, size_t ivecsize,
		    size_t *iveclen);
  };

#ifdef _cplusplus
}
#endif

#endif
