/* $Id: ofb.c,v 1.1 2002/01/24 23:24:24 levitte Exp $ */

/* ====================================================================
 * Copyright (c) 1999 Levitte Programming.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *      This product includes software developed by Levitte Programming
 *      and its contributors.
 *
 * 4. The names "RLCRYPTO" and "Levitte Programming" or the names of its
 *    contributors must not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *    For written permission, please contact levitte@lp.se.
 *
 * THIS SOFTWARE IS PROVIDED BY LEVITTE PROGRAMMING AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL LEVITTE
 * PROGRAMMING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

#include <stdlib.h>
#include <string.h>
#include "../algo_int.h"

struct algorithm_context_st
{
  unsigned char iv[64];
  size_t iv_len;
  size_t iv_offset;		/* Measured in 8-bit chunks */
};

static size_t ofb_ctx_size(void)
{
  return sizeof(ALGO_CTX);
}

static ALGO_CTX *ofb_alloc(void)
{
  return (ALGO_CTX *)malloc(ofb_ctx_size());
}

static void ofb_free(ALGO_CTX *ctx)
{
  free(ctx);
}

static int ofb_init(ALGO_CTX *ctx, const unsigned char *ivec, size_t iveclen)
{
  memset(ctx->iv, 0, sizeof(ctx->iv));
  memcpy(ctx->iv, ivec, iveclen);
  if (iveclen > 64)
    return ALGO_E_TOO_LARGE_BLOCK;
  ctx->iv_len = iveclen;
  return ALGO_E_OK;
}

static void ofb_clear(ALGO_CTX *ctx)
{
  memset(ctx, 0, ofb_ctx_size());
}

static int ofb_enc_dec(ALGO_CTX *ctx,
		       const unsigned char *input, unsigned char *output, size_t length,
		       int encrypt,
		       int (*sym_enc_dec)(ALGO_CTX *ctx,
					  const unsigned char *input,
					  unsigned char *output, size_t length,
					  int encrypt),
		       ALGO_CTX *sym_data)
{
  int status = ALGO_E_OK;
  size_t n;
  unsigned char tmp[64];
  register size_t iv_len = ctx->iv_len;

  while (length)
    {
      if (ctx->iv_offset)
	{
	  for(n=ctx->iv_offset; n < iv_len && length; n++, length--)
	    {
	      *output++ = *input++ ^ ctx->iv[n];
	    }
	  ctx->iv_offset = n % iv_len;
	}
      if (length)
	{
	  status = sym_enc_dec(sym_data, ctx->iv, tmp, iv_len,
			       OPENSSL_ENCRYPT);
	  if (status != ALGO_E_OK)
	    return status;
	  memcpy(ctx->iv, tmp, iv_len);
	  if (length < iv_len)
	    {
	      for(n=0; n < length; n++)
		{
		  *output++ = *input++ ^ tmp[n];
		}
	      ctx->iv_offset = length;
	      length = 0;
	    }
	  else
	    {
	      for(n=0; n < iv_len; n++, length--)
		{
		  *output++ = *input++ ^ tmp[n];
		}
	    }
	}
    }
  return status;
}

static int ofb_cpy_ivec(ALGO_CTX *ctx, unsigned char *ivec, size_t ivecsize,
			size_t *iveclen)
{
  if (ivecsize < ctx->iv_len)
    return ALGO_E_INVALID_IVEC_SIZE;
  memcpy(ivec, ctx->iv, ctx->iv_len);
  *iveclen = ctx->iv_len;
  return ALGO_E_OK;
}

static const MODE_METHOD ofb_method =
  {
    "OFB",
    0,
    1,
    &ofb_ctx_size,
    &ofb_alloc,
    &ofb_free,
    &ofb_init,
    &ofb_clear,
    &ofb_enc_dec,
    &ofb_cpy_ivec
  };

const MODE_METHOD *OFB(void)
{
  return &ofb_method;
}
